package com.sams.learnweblogic7.ejb.jms;

import javax.naming.*;
import javax.ejb.*;
import java.util.*;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;


public class ChefBean implements SessionBean 
{
	private String chefId;
	private static final boolean ENABLE_WEBLOGIC_LOGGING = true;
	private boolean isAvailable;
    private SessionContext ctx;
	//Defining context factory. This is the default WebLogic's context factory. This class can be found in 
	//the weblogic.jar package located in the C:\bea\weblogic700\server\lib directory.
	public static final String WEBLOGIC_JNDI_FACTORY = "weblogic.jndi.WLInitialContextFactory";

	//defining the name of the JMS connection factory which contains our queues.
	//this has to be configured via the admin console.
	public static final String JMS_FACTORY = "learnweblogic7.jms.QueueConnectionFactory";

	//defining the name of the queue to be used for sending OrderedItem Messages.
	public static final String ORDERED_ITEMS_QUEUE = "learnweblogic7.jms.orderedItemsQueue";

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Used when create is called on the remote interface
	 */
	public void ejbCreate(){
		webLogicLog("Method ejbCreate has been called");
	}
	
	// Using WebLogic's log service to log events
	private void webLogicLog(String logMessage) {
		if (ENABLE_WEBLOGIC_LOGGING) 
			System.out.println(logMessage);
	} //end of webLogicLog

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbActivate() {
		webLogicLog("Method ejbActivate has been called");
	}//end of ejbActivate

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbRemove() {
		webLogicLog("Method ejbRemove has been called");
	}//end of ejbRemove

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbPassivate() {
		webLogicLog("Method ejbPassivate has been called");
	}//end of ejbPassivate


	/**
	 * Sets the session context.
	 *
	 * @param ctx SessionContext Context for session
	 */
	public void setSessionContext(SessionContext ctx) {
		webLogicLog("Method setSessionContext has been called");
		this.ctx = ctx;
	}//end of setSessionContext

	/**
	   * Look up the bean's home interface using JNDI.
	   */
	  private ItemHome lookupHome()
		throws NamingException
	  {
		Hashtable h = new Hashtable();
		h.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
		h.put(Context.PROVIDER_URL, "t3://localhost:7001");
		Context ctx = new InitialContext(h);
		System.out.println("Initial context to look up BMP ItemHome created "+ctx);
		try {
		  Object home = (ItemHome)ctx.lookup("BMP_ItemHome");
		  return (ItemHome) PortableRemoteObject.narrow(home, ItemHome.class);
		} catch (NamingException ne) {
		  webLogicLog("Unable to lookup the EJBHome.  Verify the ejb JNDI name BMP_ItemHome on the WebLogic server");
		  throw ne;
		}
	  }


  /**
   * This method is called by the waiter to cook the item.
   *
   * @param orderedItem       MenuItem object ordered item
   * @return                  object CookedItem
   */
	public void cookItem(MenuItem orderedItem){
		ItemHome home;
		ItemPK orderedItemPk;
		String EJBQLIngredients;
		String orderedItemName;
		String orderedItemServingSize;
		String orderedItemTaste;
		String orderedItemIngredients;
		String orderedItemTemperature;
		webLogicLog("cook Item called");
		int orderedItemId = orderedItem.getItemId();
		CookedItemImpl preparedItem = new CookedItemImpl();
		System.out.println("The ordered item desc is:"+orderedItem.getItemDesc());
		//call the entity bean now to get the attributes for the cooked Item.
		try{
			home = lookupHome();
			orderedItemPk = new ItemPK(orderedItemId);
			ItemInterface orderedItemInterface = (ItemInterface)home.findByPrimaryKey(orderedItemPk);			
			System.out.println("Item interface object created"+System.currentTimeMillis());
			orderedItemName = orderedItem.getItemDesc();
			preparedItem.setItemName(orderedItemName); 
			orderedItemServingSize = orderedItemInterface.getItemServingSize();
			preparedItem.setServingSize(orderedItemServingSize);
			orderedItemTaste = orderedItemInterface.getItemTaste();
			preparedItem.setTaste(orderedItemTaste);
			orderedItemIngredients = orderedItemInterface.getItemIngredients();
			preparedItem.setIngredients(orderedItemIngredients);
			orderedItemTemperature = orderedItemInterface.getItemTemperature();
			preparedItem.setTemperature(orderedItemTemperature);
		
			InitialContext ic = getInitialContext("t3://localhost:7001");
			SendMessagesToQueue messageSender = new SendMessagesToQueue();
			messageSender.init(ic, ORDERED_ITEMS_QUEUE);
			SendMessagesToQueue.readAndSend(messageSender,preparedItem);
			messageSender.close();
	//		return preparedItem;
		}
		catch(Exception e){
			e.printStackTrace();
		}
		//changed here for JMS.
	}//end of cookItem

	private static InitialContext getInitialContext(String url)
       throws NamingException
	{
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, WEBLOGIC_JNDI_FACTORY);
		env.put(Context.PROVIDER_URL, url);
		return new InitialContext(env);
    }

}
