package com.sams.learnweblogic7.ejb.jms;

import java.io.*;
import java.util.*;
import javax.transaction.*;
import javax.naming.*;
import javax.jms.*;

public class ReceiveMessagesFromQueue implements MessageListener
{

	//Defining context factory. This is the default WebLogic's context factory. This class can be found in 
	//the weblogic.jar package located in the C:\bea\weblogic700\server\lib directory.
	public static final String WEBLOGIC_JNDI_FACTORY = "weblogic.jndi.WLInitialContextFactory";

	//defining the name of the JMS connection factory which contains our queues.
	//this has to be configured via the admin console.
	public static final String JMS_FACTORY = "learnweblogic7.jms.QueueConnectionFactory";

	//defining the name of the queue to be used for sending OrderedItem Messages.
	public static final String ORDERED_ITEMS_QUEUE = "learnweblogic7.jms.orderedItemsQueue";

	private QueueConnectionFactory qConnFactory;
	private QueueConnection qConn;
	private QueueSession qSession;
	private QueueReceiver qReceiver;
	private Queue queue;
	private boolean quit = false;
	Vector cookedItems = new Vector();
	int orderedItemsCtr=0;
	int receivedItemsCtr=0;

/**
 * Message listener interface.
 * @param msg  message
 */

  // MessageListener interface
  public void onMessage(Message msg)
  {
	  CookedItemImpl cookedItem=null;
	try {
	  if(msg instanceof ObjectMessage){
		ObjectMessage objMsg = (ObjectMessage)msg;
		cookedItem = (CookedItemImpl)objMsg.getObject();
		System.out.println("Inside receive messages, cookedItem:"+cookedItem.getItemName());
		System.out.println("Message Received:"+cookedItem);
		cookedItems.addElement(cookedItem);	
		System.out.println("******Added cooked Item "+cookedItem.getItemName());
		receivedItemsCtr++;
		System.out.println("******receivedITemCtr"+receivedItemsCtr);
		}
	  else{
		System.out.println("Error in message");
	  }
    } catch (JMSException jmse) {
      jmse.printStackTrace();
    }
  }

  /**
   * Creates all the necessary objects for receiving
   * messages from a JMS queue.
   *
   * @param   ctx	JNDI initial context
   * @param	queueName	name of queue
   * @exception NamingException if operation cannot be performed
   * @exception JMSException if JMS fails to initialize due to internal error
   */
  public void init(Context ctx, String queueName)
       throws NamingException, JMSException
  {
    qConnFactory = (QueueConnectionFactory) ctx.lookup(JMS_FACTORY);
    qConn = qConnFactory.createQueueConnection();
    qSession = qConn.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
    queue = (Queue)ctx.lookup(queueName);
    qReceiver = qSession.createReceiver(queue);
    qReceiver.setMessageListener(this);
    qConn.start();
  }

  /**
   * Closes JMS objects.
   * @exception JMSException if JMS fails to close objects due to internal error
   */
  public void close()
       throws JMSException
  {
    qReceiver.close();
    qSession.close();
    qConn.close();
  }

  public void initQueue()
       throws Exception
  {
    InitialContext ic = getInitialContext("t3://localhost:7001");
    init(ic, ORDERED_ITEMS_QUEUE);

    System.out.println("JMS Ready To Receive Messages ");
  }

  private InitialContext getInitialContext(String url)
       throws NamingException
  {
    Hashtable env = new Hashtable();
    env.put(Context.INITIAL_CONTEXT_FACTORY, WEBLOGIC_JNDI_FACTORY);
    env.put(Context.PROVIDER_URL, url);
    return new InitialContext(env);
  }

  public Vector getAllCookedItems(int itemsOrderedCtr){
  	System.out.println("Sending from Receive Messages from queue"+cookedItems);
	return cookedItems;
  }
}