package com.sams.learnweblogic7.jndi;

import java.util.*;

public class WaiterImpl implements WaiterInterface
{
	//defining private and public variables, for the list of items ordered by client, and items cooked by chef
	private Vector orderedItemsList = new Vector();
	private Vector cookedItems = new Vector();
	public String tempString = "Trial String";

	//method called by client to get a list of Menu items.
	public MenuList getMenu(){
		return MenuList.getMenu();
	}
	
	//method called by client to get the order. 
	//The client gets the menu item id from the menu list, and also specifies the number of items.
	public void getOrder(MenuItem orderedItem, int quantity){
		orderedItemsList.addElement(new Order(orderedItem, quantity));	
	}
	
	//the client calls this method to confirm his order. When he does this, the waiter requests for 
	//a chef available from the chef pool, and sends the item to be prepared to the chef.
	//The chef returns the cooked item. The waiter then collects all the cooked items into his tray (the global vector),
	//returns the cook into the cook pool.
	public void confirmOrder(){
		CookedItemImpl currentCookedItem;
		MenuItem menuItemToBeProcessed;
		Enumeration orderedItemsEnum = orderedItemsList.elements();
		while(orderedItemsEnum.hasMoreElements())
		{
			ChefInterface currentChef = getChefFromChefPool();
			menuItemToBeProcessed = ((Order)orderedItemsEnum.nextElement()).getItem();
			currentCookedItem = currentChef.cookItem(menuItemToBeProcessed);
			cookedItems.addElement(currentCookedItem);
			PoolClass.returnChef(currentChef);
		}
	}
	
	//called by the waiter to get a chef from the chef pool
	public ChefInterface getChefFromChefPool(){
		return PoolClass.getChef();
	}
	
	//performed by the waiter after all the items are cooked, and collected in the vector, and then serves the client.
	public Vector serveCookedItem(){
		return cookedItems;		
	}
	
	//called by the client, to get his check.
	public float getCheck(){
		float checkSum = 0;
		Enumeration orderedItemsEnum = orderedItemsList.elements();
		while(orderedItemsEnum.hasMoreElements())
		{
			Order individualCurrOrder = (Order)orderedItemsEnum.nextElement();
			checkSum += (individualCurrOrder.getItem()).getItemPrice()*individualCurrOrder.getQty();
		}
		return checkSum;
	}
};